#!/bin/sh
#
#	@(#) dolicense.shinc 12.5 97/11/19 
#
# Copyright (c) 1997 The Santa Cruz Operation, Inc.. All Rights Reserved.
# THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF THE SANTA CRUZ OPERATION INC.
# The copyright notice above does not evidence any actual or intended
# publication of such source code.
#


##########################################################################
#
# shell script to add a license
#
# reads the LINFO_* definitions from the l_info file passed in, 
# and based on this information:
# 1. returns, silently, as no license is required, or
#    returns, silently, as a valid license is already on the system
# 2. if -q, then run absolutely silently, and
#        if not -e then exit without any licensing having been performed
#        if -e, then if there is an eval available and valid, add it
# 3. if not -q, prompts for one of
#        add a license, use eval if available, or defer
#
# Arguments are:
#
#	[-q [-e]]	- -q - run silently, for initial system loads
#			  -e - if silent and no valid license is present
#			       but an eval is available, add it
#	l_info_path	- path to the l_info file, for LINFO_* definitions
#	menu_path	- path to the directory containing the forms -
#			  ignored if -q is given
#
# Exit codes are:
#
#	0		- success - license already present on the system,
#			            or was added by user successfully
#	1		- success - an eval license was added
#				    (automatically if -q -e)
#	2		- failure - no license was added, either by the
#				    user or automatically
#	3		- failure (on ERROR, normally)
#	other non-zero	- exit status from underlying command
#
##########################################################################

##########################################################################
#
# Format of the l_info file:
#
# Overall license control:
# 	LINFO_NOT_LICENSED	If YES, then this product requires no
#				licensing at all (and no further l_info
#				fields need be present)
#
# License map data (optional section; if these entries do not exist,
# no license map updates are attempted) - basic data:
# 	LINFO_LICMAP_ID		ID number for map entry
# 	LINFO_LICMAP_VERSION	License version number
# 	LINFO_LICMAP_DATA	Default license data to store in map
# 	LINFO_LICMAP_NAME	Name of the product associated with entry
# ...and 'bump' data (again, optional):
# 	LINFO_LICMAP_ID_B	ID number for map entry
# 	LINFO_LICMAP_VERSION_B	License version number
# 	LINFO_LICMAP_DATA_B	Default license data to store in map
# 	LINFO_LICMAP_NAME_B	Name of the product associated with entry
#
# General license control:
#	LINFO_ISLD_ARGS		Args to brand to detect if a valid product
#				license is already present (eval or real)
# 	LINFO_EVAL		If YES, the l_info file holds an eval
#				license which may be used
#
# Evaluation license information (absent if LINFO_EVAL="NO"):
#	LINFO_L_NUM		Evaluation license number
#	LINFO_L_CODE		Evaluation license code
#	LINFO_L_DATA		Evaluation license data (optional field)
#
# Encryption information:
#	LINFO_ENCRYPT_ARGS	Args to brand to detect if a strong
#				encryption license is installed
#
##########################################################################

# definitions
TRUE=1; FALSE=0
SUCCESS=0; FAIL=1
ME="`basename $0 .sh`"
USAGE="Usage: $ME [ -q [ -e ]] l_info_path menu_path"
CCSTMP="/tmp/ccs_$$"
BRAND="/usr/sbin/brand"
LICENSE_MGR="/usr/lib/scoadmin/license/licenseMgr"
MESSAGES_DEFS="dolicense.msgs"
RETURN_COVERED=0; RETURN_ADDED_EVAL=1
RETURN_NOT_LICENSED=2; RETURN_FAIL=3

# included standard routines
##########################################################################
#
#	@(#) test_license.sh_h 11.5 97/10/15 
#
# Standard routine to check if a license is already covered by something
# installed on the system
#
# Arguments are:
#	$1	- y-bit flags values to test for - this is a set of
#		  y-values, separated by colons.  If any match then
#		  the product is considered licensed
#
# Return values are:
#	$SUCCESS- license is already covered by something else
#	$FAIL	- no license covers this product
#
##########################################################################
test_license()
{
	test_string="$1"

	# elements in $1 are separated by colons - test each element
	# to see if the licensing indicated by them is covered; if it
	# is, we can return $SUCCESS, else we return $FAIL
	for license_string in `echo "$test_string" | tr ' :' ': '`; do
		license_arg="`echo \"$license_string\" | tr ':' ' '`"

		# pass the args string as an argument (in its totality)
		# to brand; if brand returns zero then this license
		# check succeeded (the license is covered)
		/usr/sbin/brand $license_arg >/dev/null 2>&1
		if [ $? -eq 0 ]; then
			return $SUCCESS
		fi
	done

	# no licensing element is covered, so return $FAIL
	return $FAIL
}



# trap to ignore signals
trap '' 1 2 3 15

# arguments for directories and file definitions
QUIET=$FALSE; EVAL_ACCEPTABLE=$FALSE
while getopts "qe" opt; do
	case "$opt" in
	'q')	QUIET=$TRUE		;;
	'e')	EVAL_ACCEPTABLE=$TRUE	;;
	'?')	echo "$USAGE" >&2
		exit $RETURN_FAIL		;;
	esac
done
shift `expr $OPTIND - 1`
if [ $# -ne 2 ]; then
	echo "$USAGE" >&2
	exit $RETURN_FAIL
fi

L_INFO_PATH="$1"
if [ "`echo \"$L_INFO_PATH\" | cut -c1`" != '/' ]; then
	# not absolute, so add pwd to make absolute
	L_INFO_PATH="`pwd`/$L_INFO_PATH"
fi
MENU_PATH="$2"
if [ "`echo \"$MENU_PATH\" | cut -c1`" != '/' ]; then
	# not absolute, so add pwd to make absolute
	MENU_PATH="`pwd`/$MENU_PATH"
fi

# set up our path
PATH=/bin:/usr/bin:/sbin:/usr/sbin:/etc
export PATH

# read message definitions for MSG_EVAL_MSG, MSG_MUST_LICENSE, MSG_BAD_EVAL,
# and MSG_DUP_EVAL
if [ -f $MENU_PATH/$MESSAGES_DEFS ]; then
	. $MENU_PATH/$MESSAGES_DEFS
else
	echo "$ME: internal error - no $MESSAGES_DEFS file found" >&2
	exit $RETURN_FAIL
fi

# symbols to help find menus and menu processing
MENU_COLORS="menu_colors"
ERROR_COLOR="error"
WARNING_COLOR="warn"
REGULAR_COLOR="regular"
STD_INDENT="        "
MENU="menu"
MENU_TMPFILE="/tmp/dolicense.menu$$"
INFO_MENU="menu.dolicenseinfo"
WARNING_MENU="menu.dolicensewarning"
ERROR_MENU="menu.dolicenseerror"
EVAL_REAL_OR_DEFER_MENU="menu.dolicense01"
REAL_OR_DEFER_MENU="menu.dolicense02"

# if -q then no menu stuff is required
LOCALE="${LC_ALL:-${LC_MESSAGES:-${LANG:-C}}}"
if [ $QUIET -ne $TRUE ]; then
	# set up to use menu_colors; default to C-locale if locale's dir has
	# no menu_colors.sh
	if [ -f /etc/inst/locale/$LOCALE/menus/menu_colors.sh ]
	then
		. /etc/inst/locale/$LOCALE/menus/menu_colors.sh
	else
		if [ -f /etc/inst/locale/C/menus/menu_colors.sh ]; then
			. /etc/inst/locale/C/menus/menu_colors.sh
		else
			echo "$ME: internal error - no /etc/inst/locale/C/menus/menu_colors.sh file found" >&2
			exit $RETURN_FAIL
		fi
	fi
fi


##########################################################################
#
# fatal error function
#
##########################################################################
dolicense_fatal()
{
	# print out the fatal error message and exit
	echo "\n$1" >&2

	# exit
	exit $RETURN_FAIL
} # dolicense_fatal()


##########################################################################
#
# Menu forms library function
# Arguments are
#	$1 - name of the form to run
#	$2 - file to send form (menu) -o output to
#	$3 - color set to use (regular, warn, or error)
#	$4 - string to eval as forms arguments
#
##########################################################################
run_form()
{
	# runtime check of mode - are forms valid?
	if [ $QUIET -ne $TRUE ]; then

		# catch function parameters
		form_name="$1"
		output_file="$2"
		color="$3"
		form_args="$4"

		# set requested menu colors
		$MENU_COLORS $color

		# find the form that we have been asked to run; complain if
		# not there
		form_path="$MENU_PATH/$form_name"
		if [ ! -f "$form_path" ]; then
			dolicense_fatal "$ME: internal error - no form $form_name"
		fi

		# build a form header line
		form_header="$ME`[ -n "$PKGNAME" ] && echo \" ($PKGNAME)\"`"

		# run the form and send output to $2 parameter
		eval "FORM_HEADER=\"$form_header\" $form_args \
				$MENU $form_opts -f $form_path -o $output_file"

	else

		# forms should not be being called
		dolicense_fatal "$ME: internal error - run_form() called in -q mode"
	fi
} # run_form()


##########################################################################
#
# Menu info dialog convenience function.  Arguments are
#	$1 - message text
#
##########################################################################
info_dialog()
{
	if [ $QUIET -ne $TRUE ]; then
		msg="`echo \"$1\" | sed -e \"s/^/${STD_INDENT}/\"`"
		run_form "$INFO_MENU" "/dev/null" "$REGULAR_COLOR" \
				"MESSAGE=\"$msg\""
	else
		echo "\n\n$1\n"
	fi
} # info_dialog()


##########################################################################
#
# Menu warning dialog convenience function.  Arguments are
#       $1 - message text
#
##########################################################################
warning_dialog()
{
	if [ $QUIET -ne $TRUE ]; then
		msg="`echo \"$1\" | sed -e \"s/^/${STD_INDENT}/\"`"
		run_form "$WARNING_MENU" "/dev/null" "$WARNING_COLOR" \
				"MESSAGE=\"$msg\""
	else
		echo "\n\n$1\n" >&2
	fi
} # warning_dialog()


##########################################################################
#
# Menu error dialog convenience function.  Arguments are
#       $1 - message text
#
##########################################################################
error_dialog()
{
	if [ $QUIET -ne $TRUE ]; then
		msg="`echo \"$1\" | sed -e \"s/^/${STD_INDENT}/\"`"
		run_form "$ERROR_MENU" "/dev/null" "$ERROR_COLOR" \
				"MESSAGE=\"$msg\""
	else
		echo "\n\n$1\n" >&2
	fi
} # error_dialog()


##########################################################################
#
# license_it
#
# Arguments are:
# 	$1	- "real" then run scolicense
#		  "eval" then add a license with eval data
#
# Return values are:
#	(selection of RETURN_ codes)
#
##########################################################################
license_it()
{
	# decide what form of license to add based on the first argument
	case "$1" in 
	"real")	if [ $QUIET -eq $FALSE ]; then
			if [ -x "$LICENSE_MGR" ]; then
				# run the license manager, in CHARM mode
				# rather than X, so that everything happens
				# in the one installation window
				CHARM="yes" $LICENSE_MGR
				if [ $? -ne 0 ]; then
					warning_dialog "$MSG_LMGR_ERROR"
					return $RETURN_NOT_LICENSED
				fi

				# check to see if correct licensing is
				# now there
				test_license "$LINFO_ISLD_ARGS"
				if [ $? -eq $SUCCESS ]; then
					# all covered
					info_dialog "$MSG_OK_LICENSE"
					return $RETURN_COVERED
				else
					# still no license there
					warning_dialog "$MSG_BAD_LICENSE"
					return $RETURN_NOT_LICENSED
				fi
			else
				# can't run the license manager
				warning_dialog "$MSG_LMGR_ERROR"
				return $RETURN_NOT_LICENSED
			fi
		else
			# shouldn't be called with "real" if -q
			warning_dialog "$MSG_MUST_LICENSE"
			return $RETURN_NOT_LICENSED
		fi
		;;

	"eval")	# add the license, with license data if supplied
		if [ -n "$LINFO_L_DATA" ]; then
			$BRAND -g -a "$LINFO_L_DATA" "$LINFO_L_NUM" \
					"$LINFO_L_CODE" >/dev/null 2>$CCSTMP
		else
			$BRAND -g "$LINFO_L_NUM" "$LINFO_L_CODE" \
					>/dev/null 2>$CCSTMP
		fi
		branded=$?
		case $branded in
		0)
			# confirm success
			info_dialog "$MSG_EVAL_MSG"
			return_code=$RETURN_ADDED_EVAL
			;;
		2)
			# try to report bad eval license
			error_dialog "$MSG_BAD_EVAL"
			return_code=$RETURN_NOT_LICENSED
			;;
		10)
			# try to report duplicate eval license
			error_dialog "$MSG_DUP_EVAL"
			return_code=$RETURN_COVERED
			;;
		*)
			dolicense_fatal "$ME: internal error - brand returned unknown status $branded"
			;;
		esac
		if [ $branded -ne 0 -a -s $CCSTMP ]; then
			error_dialog "$BRAND stderr:\n\n`cat $CCSTMP`"
		fi
		rm $CCSTMP 2>/dev/null
		return $return_code
		;;

	*)	dolicense_fatal "$ME: internal error - invalid argument: $1"
		;;
	esac

	# should never get here
	return $RETURN_FAIL
}


##########################################################################
#
# main
#
##########################################################################

# if we are called from pkgadd then PKG will be set to the id of the
# package, and NAME will be set to the package name - trap these so
# they can be used in forms to remind the user what exactly they are
# installing
PKGNAME=""
if [ -n "$PKG" ]; then
	PKGNAME="$NAME"
fi

# then read in the l_info file, to get the LINFO_* definitions
if [ -f "$L_INFO_PATH" ]; then
	. $L_INFO_PATH
else
	dolicense_fatal "$ME: can't find file $L_INFO_PATH"
fi

# if no license required, exit
fc="`echo \"$LINFO_NOT_LICENSED\" | cut -c1`"
if [ "$fc" = "Y" -o "$fc" = "y" ]; then
	 exit $RETURN_COVERED
fi

# ...or if the licensing requirements are already covered then also
# exit
test_license "$LINFO_ISLD_ARGS"
if [ $? -eq $SUCCESS ]; then
	exit $RETURN_COVERED
fi

# if we are here, then no license valid for this product was found in
# the system; so we may have to add some sort of license

# we're about to license; register the product with the license map
# if we have license map data in the l_info file
if [ -n "$LINFO_LICMAP_ID" -a -n "$LINFO_LICMAP_VERSION" \
	-a -n "$LINFO_LICMAP_DATA" -a -n "$LINFO_LICMAP_NAME" ]; then

	$BRAND -p "$LINFO_LICMAP_ID" "$LINFO_LICMAP_VERSION" \
		"$LINFO_LICMAP_DATA" "$LINFO_LICMAP_NAME" \
				>/dev/null 2>&1
	if [ $? -ne 0 ]; then
		# license map update failed - could be that the map
		# entry is in use
		error_dialog "$MSG_LICMAP_ERROR"
		exit $RETURN_NOT_LICENSED
	fi
fi
#... and if we have secondary license map data then add that too
if [ -n "$LINFO_LICMAP_ID_B" -a -n "$LINFO_LICMAP_VERSION_B" \
	-a -n "$LINFO_LICMAP_DATA_B" -a -n "$LINFO_LICMAP_NAME_B" ]; then

	$BRAND -p "$LINFO_LICMAP_ID_B" "$LINFO_LICMAP_VERSION_B" \
		"$LINFO_LICMAP_DATA_B" "$LINFO_LICMAP_NAME_B" \
				>/dev/null 2>&1
	if [ $? -ne 0 ]; then
		# license map update failed - could be that the map
		# entry is in use
		error_dialog "$MSG_LICMAP_ERROR_B"
		exit $RETURN_NOT_LICENSED
	fi
fi

# now, check if eval is possible and if possible, and if in interactive
# mode, then prompt to see if the user has a real COLA to add, will use
# the eval one, or will leave it for later

# if eval is possible and we are in quiet mode, we have a choice - either
# add the eval license, or leave it for later.  The -e flag controls
# which we do.

# if the eval license had been previously used but not expired, then
# ISLD above would have caught it and accepted it

# if the license has previously been used but has expired, then brand
# will exit 2

# is evaluation license possible? - does the l_info file claim to contain
# an evaluation license, and can we see a non-blank number and code?
fc="`echo \"$LINFO_EVAL\" | cut -c1`"
if [ \( "$fc" = "Y" -o "$fc" = "y" \) \
		-a -n "$LINFO_L_NUM" -a -n "$LINFO_L_CODE" ]; then

	# loop until a valid license is added or we give up
	for i in 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15; do

		# if we can ask questions then ask whether we should license
		# for evaluation, license for real, or not license at all
		if [ $QUIET -eq $FALSE ]; then
			run_form "$EVAL_REAL_OR_DEFER_MENU" "$MENU_TMPFILE" \
					"$REGULAR_COLOR" ""
			if [ -f $MENU_TMPFILE ]; then
				. $MENU_TMPFILE; rm $MENU_TMPFILE
			else
				RETURN_VALUE=3
			fi
			ans=$RETURN_VALUE
		else

			# we can ask nothing, so forge a response of 'eval' if
			# -e was given, or 'defer' if not.  We certainly cannot
			# prompt for any real license at this point
			if [ $EVAL_ACCEPTABLE = $TRUE ]; then
				ans=1
			else
				ans=3
			fi
		fi

		# now license with either eval or real, or don't license at all
		case $ans in
		1)				# eval license
			license_it eval
			exit $?
			;;
		2)				# normal license
			license_it real
			returncode=$?
			if [ $returncode -eq $RETURN_NOT_LICENSED ]; then
				# user failed to enter a valid license - retry
				continue
			else
				exit $returncode
			fi
			;;
		3)				# skipped licensing altogether
			warning_dialog "$MSG_MUST_LICENSE"
			exit $RETURN_NOT_LICENSED
			;;
		*)				# error
			dolicense_fatal "$ME: internal error - bad ans: $ans"
			;;
		esac
	done
	warning_dialog "$MSG_MUST_LICENSE"
	exit $RETURN_NOT_LICENSED

else
	# this product has no eval license included

	# loop until a valid license is added or we give up
	for i in 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15; do

		# if we can ask questions then ask whether we should license
		# for real, or not license at all
		if [ $QUIET -eq $FALSE ]; then
			run_form "$REAL_OR_DEFER_MENU" "$MENU_TMPFILE" \
					"$REGULAR_COLOR" ""
			if [ -f $MENU_TMPFILE ]; then
				. $MENU_TMPFILE; rm $MENU_TMPFILE
			else
				RETURN_VALUE=2
			fi
			ans=$RETURN_VALUE
		else

			# we can ask nothing, and we don't have an eval license
			# available to us.  Regardless of the -e flag, we have
			# to defer at this point
			ans=2
		fi

		# now license with real, or don't license at all
		case $ans in
		1)				# normal license
			license_it real
			returncode=$?
			if [ $returncode -eq $RETURN_NOT_LICENSED ]; then
				# user failed to enter a valid license - retry
				continue
			else
				exit $returncode
			fi
			;;
		2)				# skipped licensing altogether
			warning_dialog "$MSG_MUST_LICENSE"
			exit $RETURN_NOT_LICENSED
			;;
		*)				# error
			dolicense_fatal "$ME: internal error - bad ans: $ans"
			;;
		esac
	done
	warning_dialog "$MSG_MUST_LICENSE"
	exit $RETURN_NOT_LICENSED
fi
dolicense_fatal "$ME: internal error - no valid termination condition"
